drop view if exists public.view_disciplines_stat;

DROP FUNCTION IF EXISTS public.disciplinecollection_get(psemesterid integer, pfacultyid integer);
DROP FUNCTION IF EXISTS public.disciplinecollection_get(psemesterid integer, pfacultyid integer, pgradenum integer, pdegree character varying);
DROP FUNCTION IF EXISTS public.disciplinecollection_get(psemesterid integer, pfacultyid integer, pgradenum integer, pdegree bachelor_master_specialist);
CREATE OR REPLACE FUNCTION public.disciplinecollection_get(psemesterid integer, pfacultyid integer, pgradenum integer default null, pdegree bachelor_master_specialist default null)
    RETURNS TABLE("ID" integer, "SubjectName" character varying, "AuthorID" integer, "State" text, "IsInactive" boolean, "PlanExternalID" integer, "GlobalName" character varying)
    LANGUAGE plpgsql
AS $function$
    -- select public.discipline_getwithfullness(
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
-- );
BEGIN
    return query
        SELECT view_disciplines.DisciplineID AS ID,
               view_disciplines.SubjectName As SubjectName,
               view_disciplines.AuthorID As AuthorID,
               CASE
                   WHEN (view_disciplines.MaxRate < 100 AND view_disciplines.IsLocked = 0) THEN 'no_map'
                   WHEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 0) THEN 'no_rates'
                   WHEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 1) THEN 'in_progress'
                   ELSE 'other'
                   END AS State,
               disciplines.IsInactive as IsInactive,
               view_disciplines.planexternalid,
               view_disciplines.globalname as GlobalName
        FROM view_disciplines
                 JOIN disciplines ON view_disciplines.disciplineid = disciplines.id
        WHERE view_disciplines.SemesterID = pSemesterID AND
                view_disciplines.FacultyID = pFacultyID AND
            (view_disciplines.GradeNum = pgradenum or ((public.view_disciplines.GradeNum = pgradenum)is unknown)) AND
            (view_disciplines.Degree = pdegree or ((public.view_disciplines.Degree = pdegree)is unknown));
END
$function$;

CREATE OR REPLACE VIEW public.view_disciplines_stat
AS SELECT ff.name,
          ( SELECT count(*) AS count
            FROM disciplinecollection_get(11, ff.id) disciplinecollection_get("ID", "SubjectName", "AuthorID", "State")) AS "Всего дисциплин",
          ( SELECT count(*) AS count
            FROM disciplinecollection_get(11, ff.id) t("ID", "SubjectName", "AuthorID", "State", "IsInactive")
            WHERE t."State" = 'no_map'::text AND t."IsInactive" = false) AS "Без УКД"
   FROM faculties ff;

CREATE OR REPLACE VIEW public.view_disciplines
AS SELECT distinct on (disciplines.id)
       disciplines.id AS disciplineid,
       disciplines.authorid,
       disciplines.examtype,
       disciplines.lecturecount,
       disciplines.practicecount,
       disciplines.labcount,
       disciplines.semesterid,
       disciplines.islocked,
       disciplines.milestone,
       disciplines.subtype,
       disciplines.compounddiscid,
       disciplines.maxrate,
       disciplines.currate,
       disciplines.isinactive,
       disciplines.isglobal,
       grades.id AS gradeid,
       grades.num AS gradenum,
       grades.degree,
       subjects.id AS subjectid,
       subjects.name AS subjectname,
       subjects.abbr AS subjectabbr,
       faculties.id AS facultyid,
       faculties.name AS facultyname,
       faculties.abbr AS facultyabbr,
       compound_disciplines.name AS compounddiscname,
       global_disciplines.name AS globalname,
       study_plans.externalid AS planexternalid
   FROM disciplines
            JOIN subjects ON subjects.id = disciplines.subjectid
            JOIN faculties ON faculties.id = disciplines.facultyid
            LEFT JOIN grades ON grades.id = disciplines.gradeid
            LEFT JOIN compound_disciplines ON compound_disciplines.id = disciplines.compounddiscid
            LEFT JOIN disciplines_study_plans on disciplines_study_plans.disciplineid = disciplines.id
            LEFT JOIN study_plans ON study_plans.id = disciplines_study_plans.studyplanid
            LEFT JOIN global_disciplines ON global_disciplines.disciplineid = disciplines.id;




