CREATE SEQUENCE seq_tasks
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

select setval('seq_tasks', 1, true);

CREATE TABLE tasks
(
    id           integer DEFAULT nextval('seq_tasks'::regclass) NOT NULL,
    name         character varying(50)                          NOT NULL,
    recordbookid integer                                        NOT NULL,
    disciplineid integer                                        NOT NULL,
    modulenumber integer                                        NOT NULL,
    open         bool    DEFAULT true                           NOT NULL
);

ALTER TABLE ONLY tasks
    ADD CONSTRAINT tasks_name_recordbook_discipline_key UNIQUE (name, recordbookid, disciplineid);

ALTER TABLE ONLY tasks
    ADD CONSTRAINT tasks_recordbook_discipline_modulenumber_key UNIQUE (recordbookid, disciplineid, modulenumber);

ALTER TABLE ONLY tasks
    ADD CONSTRAINT task_ibfk_1 FOREIGN KEY (recordbookid) REFERENCES record_books (id);

ALTER TABLE ONLY tasks
    ADD CONSTRAINT task_ibfk_2 FOREIGN KEY (disciplineid) REFERENCES disciplines (id);

DROP FUNCTION IF EXISTS public.CheckTaskOpen(pdiscipline integer, pmodulenumber integer, precordbook integer);
CREATE OR REPLACE FUNCTION public.CheckTaskOpen(pdiscipline integer, pmodulenumber integer, precordbook integer)
    RETURNS bool
    LANGUAGE plpgsql
AS $function$
DECLARE vTaskOpen bool DEFAULT false;
BEGIN
    SELECT tasks."open" into vTaskOpen
    FROM tasks WHERE
            tasks.disciplineid = pdiscipline AND
            tasks.modulenumber = pmodulenumber AND
            tasks.recordbookid = precordbook;
    if vTaskOpen then
        return true;
    else
        return false;
    end if;
EXCEPTION
    when others
        then RETURN false;
end
$function$;

CREATE OR REPLACE FUNCTION public.closetaskfornickname(pdiscipline integer, pmodulenumber integer, nickname character varying, pservicename character varying)
    RETURNS integer
    LANGUAGE plpgsql
AS $function$
DECLARE vRecordBookID integer DEFAULT NULL;
BEGIN

    SELECT  record_books.ID into vRecordBookID
    FROM record_books
             JOIN record_books_plans on record_books.id = record_books_plans.recordbookid
             JOIN disciplines_study_plans on disciplines_study_plans.studyplanid = record_books_plans.studyplanid
             JOIN disciplines on disciplines.id = disciplines_study_plans.disciplineid
             JOIN subjects on disciplines.subjectid = subjects.id
             JOIN students on record_books.studentid = students.id
             JOIN accounts on accounts.id = students.accountid
             JOIN nicknames on nicknames.accountid = accounts.id
    WHERE disciplines.id = pdiscipline
      AND nicknames.name = nickname
      AND nicknames."servicename" = pservicename
    LIMIT 1;

    UPDATE tasks set open=false
    WHERE modulenumber = pmodulenumber AND
            recordbookid = vRecordBookID AND
            disciplineid = pdiscipline;
    return 0;
EXCEPTION
    when others
        then RETURN -1;
end
$function$
;
