CREATE SEQUENCE seq_logs_discipline_deactivate
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

DROP TABLE IF EXISTS logs_discipline_deactivate;
CREATE TABLE logs_discipline_deactivate (
                                  id integer DEFAULT nextval('seq_logs_discipline_deactivate'::regclass) NOT NULL,
                                  discipline_id integer NOT NULL,
                                  account_id integer NOT NULL,
                                  isActive bool DEFAULT FALSE NOT NULL,
                                  date timestamp(0) without time zone DEFAULT now() NOT NULL
);

CREATE INDEX key_logs_discipline_deactivate ON logs_discipline_deactivate USING btree (discipline_id);

ALTER TABLE ONLY logs_discipline_deactivate
    ADD CONSTRAINT logs_discipline_deactivate_pkey PRIMARY KEY (id);

ALTER TABLE ONLY logs_discipline_deactivate
    ADD CONSTRAINT logs_discipline_deactivate_ibfk_1 FOREIGN KEY (discipline_id) REFERENCES disciplines(id);

ALTER TABLE ONLY logs_discipline_deactivate
    ADD CONSTRAINT logs_discipline_deactivate_ibfk_2 FOREIGN KEY (account_id) REFERENCES accounts(id);

DROP FUNCTION IF EXISTS public.LogDisciplineDeactivation(paccountid integer, pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.LogDisciplineDeactivation(paccountid integer, pdisciplineid integer)
    RETURNS integer
    LANGUAGE plpgsql
AS $function$
begin
    INSERT INTO logs_discipline_deactivate (discipline_id, account_id, isActive) VALUES (pdisciplineid, paccountid, FALSE);
    RETURN 0;
END
$function$;

DROP FUNCTION IF EXISTS public.LogDisciplineActivation(paccountid integer, pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.LogDisciplineActivation(paccountid integer, pdisciplineid integer)
    RETURNS integer
    LANGUAGE plpgsql
AS $function$
begin
    INSERT INTO logs_discipline_deactivate (discipline_id, account_id, isActive) VALUES (pdisciplineid, paccountid, TRUE);
    RETURN 0;
END
$function$;